use crate::dghv::{encrypt_bit, generate_secret_key};
use rug::Integer;

pub struct PublicKey {
    pub xs: Vec<Integer>,
}

pub struct PrivateKey {
    pub p: Integer,
}

pub fn generate_keys(gamma: u32, eta: u32, rho: u32, theta: usize) -> (PrivateKey, PublicKey) {
    let p: Integer = generate_secret_key(eta);

    let mut xs: Vec<Integer> = Vec::with_capacity(theta);
    for _ in 0..theta {
        let xi = encrypt_bit(0u8, &p, gamma, rho);
        xs.push(xi);
    }
    xs.sort_by(|a, b| b.cmp(a));

    while xs[0].is_even() {
        xs.remove(0);
        let xi = encrypt_bit(0u8, &p, gamma, rho);
        xs.push(xi);
        xs.sort_by(|a, b| b.cmp(a));
    }

    (PrivateKey { p }, PublicKey { xs })
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_key_generation_properties() {
        let eta: u32 = 64;
        let gamma: u32 = 80;
        let theta: usize = 10;
        let rho: u32 = 16;
        let (sk, pk) = generate_keys(gamma, eta, rho, theta);

        assert_eq!(sk.p.significant_bits(), eta);
        assert!(sk.p.is_odd());

        assert_eq!(pk.xs.len(), theta);
        assert!(pk.xs[0].is_odd());

        for xi in pk.xs.iter().skip(1) {
            assert!(pk.xs[0] >= *xi);
        }
    }
}
